@@ -26,8 +26,6 @@ through the code to commit the whitespace formatting.
   FormatDecisionState: main class exported by this module.
 """
 
-import copy
-
 from yapf.yapflib import format_token
 from yapf.yapflib import split_penalty
 from yapf.yapflib import style
@@ -80,11 +78,21 @@ class FormatDecisionState(object):
     self.newline = False
     self.previous = None
     self.column_limit = style.Get('COLUMN_LIMIT')
-    self._MoveStateToNextToken()
 
   def Clone(self):
-    new = copy.copy(self)
-    new.stack = [copy.copy(state) for state in self.stack]
+    """Clones a FormatDecisionState object."""
+    new = FormatDecisionState(self.line, self.first_indent)
+    new.next_token = self.next_token
+    new.column = self.column
+    new.line = self.line
+    new.paren_level = self.paren_level
+    new.start_of_line_level = self.start_of_line_level
+    new.lowest_level_on_line = self.lowest_level_on_line
+    new.ignore_stack_for_comparison = self.ignore_stack_for_comparison
+    new.first_indent = self.first_indent
+    new.newline = self.newline
+    new.previous = self.previous
+    new.stack = [state.Clone() for state in self.stack]
     return new
 
   def __eq__(self, other):
@@ -95,9 +103,9 @@ class FormatDecisionState(object):
             self.column == other.column and
             self.paren_level == other.paren_level and
             self.start_of_line_level == other.start_of_line_level and
-            self.lowest_level_on_line == other.lowest_level_on_line and (
-                self.ignore_stack_for_comparison or
-                other.ignore_stack_for_comparison or self.stack == other.stack))
+            self.lowest_level_on_line == other.lowest_level_on_line and
+            (self.ignore_stack_for_comparison or
+             other.ignore_stack_for_comparison or self.stack == other.stack))
 
   def __ne__(self, other):
     return not self == other
@@ -111,13 +119,28 @@ class FormatDecisionState(object):
             (self.column, repr(self.next_token), self.paren_level,
              '\n\t'.join(repr(s) for s in self.stack) + ']'))
 
-  def CanSplit(self):
-    """Returns True if the line can be split before the next token."""
+  def CanSplit(self, must_split):
+    """Determine if we can split before the next token.
+
+    Arguments:
+      must_split: (bool) A newline was required before this token.
+
+    Returns:
+      True if the line can be split before the next token.
+    """
     current = self.next_token
 
     if current.is_pseudo_paren:
       return False
 
+    if (not must_split and
+        format_token.Subtype.DICTIONARY_KEY_PART in current.subtypes and
+        format_token.Subtype.DICTIONARY_KEY not in current.subtypes and
+        not style.Get('ALLOW_MULTILINE_DICTIONARY_KEYS')):
+      # In some situations, a dictionary may be multiline, but pylint doesn't
+      # like it. So don't allow it unless forced to.
+      return False
+
     return current.can_break_before
 
   def MustSplit(self):
@@ -131,13 +154,30 @@ class FormatDecisionState(object):
     if current.must_break_before:
       return True
 
-    if (previous and (style.Get('DEDENT_CLOSING_BRACKETS') or
-                      style.Get('SPLIT_BEFORE_FIRST_ARGUMENT'))):
+    if not previous:
+      return False
+
+    if self.stack[-1].split_before_closing_bracket and current.value in '}]':
+      # Split before the closing bracket if we can.
+      return current.node_split_penalty != split_penalty.UNBREAKABLE
+
+    # Prevent splitting before the first argument in compound statements
+    # with the exception of function declarations.
+    if (style.Get('SPLIT_BEFORE_FIRST_ARGUMENT') and
+        self.line.first.value != 'def' and
+        self.line.first.value in _COMPOUND_STMTS):
+      return False
+
+    ###########################################################################
+    # List Splitting
+    if (style.Get('DEDENT_CLOSING_BRACKETS') or
+        style.Get('SPLIT_BEFORE_FIRST_ARGUMENT')):
       bracket = current if current.ClosesScope() else previous
       if format_token.Subtype.SUBSCRIPT_BRACKET not in bracket.subtypes:
         if bracket.OpensScope():
           if style.Get('COALESCE_BRACKETS'):
             if current.OpensScope():
+              # Prefer to keep all opening brackets together.
               return False
 
           if (not _IsLastScopeInLine(bracket) or
@@ -147,23 +187,53 @@ class FormatDecisionState(object):
             last_token = _LastTokenInLine(bracket.matching_bracket)
 
           if not self._FitsOnLine(bracket, last_token):
+            # Split before the first element if the whole list can't fit on a
+            # single line.
             self.stack[-1].split_before_closing_bracket = True
             return True
 
         elif style.Get('DEDENT_CLOSING_BRACKETS') and current.ClosesScope():
+          # Split before and dedent the closing bracket.
           return self.stack[-1].split_before_closing_bracket
 
-    if self.stack[-1].split_before_closing_bracket and current.value in '}]':
-      # Split if we need to split before the closing bracket.
-      return current.node_split_penalty != split_penalty.UNBREAKABLE
+    if (current.is_name or current.is_string) and previous.value == ',':
+      # If the list has function calls in it and the full list itself cannot
+      # fit on the line, then we want to split. Otherwise, we'll get something
+      # like this:
+      #
+      #     X = [
+      #         Bar(xxx='some string',
+      #             yyy='another long string',
+      #             zzz='a third long string'), Bar(
+      #                 xxx='some string',
+      #                 yyy='another long string',
+      #                 zzz='a third long string')
+      #     ]
+      #
+      # or when a string formatting syntax.
+      func_call_or_string_format = False
+      if current.is_name:
+        tok = current.next_token
+        while tok and (tok.is_name or tok.value == '.'):
+          tok = tok.next_token
+        func_call_or_string_format = tok and tok.value == '('
+      elif current.is_string:
+        tok = current.next_token
+        while tok and tok.is_string:
+          tok = tok.next_token
+        func_call_or_string_format = tok and tok.value == '%'
+      if func_call_or_string_format:
+        open_bracket = unwrapped_line.IsSurroundedByBrackets(current)
+        if open_bracket and open_bracket.value in '[{':
+          if not self._FitsOnLine(open_bracket, open_bracket.matching_bracket):
+            return True
 
-    if not previous:
-      return False
-
-    # TODO(morbo): This should be controlled with a knob.
-    if (format_token.Subtype.DICTIONARY_KEY in current.subtypes and
+    ###########################################################################
+    # Dict/Set Splitting
+    if (style.Get('EACH_DICT_ENTRY_ON_SEPARATE_LINE') and
+        format_token.Subtype.DICTIONARY_KEY in current.subtypes and
         not current.is_comment):
-      # Place each dictionary entry on its own line.
+      # Place each dictionary entry onto its own line.
       if previous.value == '{' and previous.previous_token:
         opening = _GetOpeningBracket(previous.previous_token)
         if (opening and opening.value == '(' and opening.previous_token and
@@ -173,35 +243,60 @@ class FormatDecisionState(object):
             return False
       return True
 
-    # TODO(morbo): This should be controlled with a knob.
-    if format_token.Subtype.DICT_SET_GENERATOR in current.subtypes:
+    if (style.Get('SPLIT_BEFORE_DICT_SET_GENERATOR') and
+        format_token.Subtype.DICT_SET_GENERATOR in current.subtypes):
+      # Split before a dict/set generator.
       return True
 
+    if (format_token.Subtype.DICTIONARY_VALUE in current.subtypes or
+        (previous.is_pseudo_paren and previous.value == '(' and
+         not current.is_comment)):
+      # Split before the dictionary value if we can't fit every dictionary
+      # entry on its own line.
+      if not current.OpensScope():
+        opening = _GetOpeningBracket(current)
+        if not self._EachDictEntryFitsOnOneLine(opening):
+          return True
+
+    if previous.value == '{':
+      # Split if the dict/set cannot fit on one line and ends in a comma.
+      closing = previous.matching_bracket
+      if (not self._FitsOnLine(previous, closing) and
+          closing.previous_token.value == ','):
+        self.stack[-1].split_before_closing_bracket = True
+        return True
+
+    ###########################################################################
+    # Argument List Splitting
     if (style.Get('SPLIT_BEFORE_NAMED_ASSIGNS') and not current.is_comment and
         format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN_ARG_LIST in
         current.subtypes):
       if (previous.value not in {'=', ':', '*', '**'} and
-          current.value not in ':=,)'):
+          current.value not in ':=,)' and not _IsFunctionDefinition(previous)):
         # If we're going to split the lines because of named arguments, then we
         # want to split after the opening bracket as well. But not when this is
-        # part of function definition.
-        if not _IsFunctionDefinition(previous):
+        # part of a function definition.
+        if previous.value == '(':
           # Make sure we don't split after the opening bracket if the
           # continuation indent is greater than the opening bracket:
           #
           #  a(
           #      b=1,
           #      c=2)
-          indent_amt = self.stack[-1].indent * style.Get('INDENT_WIDTH')
-          pptoken = previous.previous_token
-          opening_column = len(pptoken.value) if pptoken else 0 - indent_amt - 1
-          if previous.value == '(':
-            return opening_column >= style.Get('CONTINUATION_INDENT_WIDTH')
-          opening = _GetOpeningBracket(current)
-          if opening:
-            arglist_length = (opening.matching_bracket.total_length -
-                              opening.total_length + self.stack[-1].indent)
-            return arglist_length > self.column_limit
+          if (self._FitsOnLine(previous, previous.matching_bracket) and
+              unwrapped_line.IsSurroundedByBrackets(previous)):
+            # An argument to a function is a function call with named
+            # assigns.
+            return False
+
+          column = self.column - self.stack[-1].last_space
+          return column > style.Get('CONTINUATION_INDENT_WIDTH')
+
+        opening = _GetOpeningBracket(current)
+        if opening:
+          arglist_length = (opening.matching_bracket.total_length -
+                            opening.total_length + self.stack[-1].indent)
+          return arglist_length > self.column_limit
 
     if style.Get('SPLIT_ARGUMENTS_WHEN_COMMA_TERMINATED'):
       # Split before arguments in a function call or definition if the
@@ -212,21 +307,51 @@ class FormatDecisionState(object):
           if opening.matching_bracket.previous_token.value == ',':
             return True
 
-    if (format_token.Subtype.DICTIONARY_VALUE in current.subtypes or
-        (previous.is_pseudo_paren and previous.value == '(')):
-      if not current.OpensScope():
-        opening = _GetOpeningBracket(current)
-        if previous.is_pseudo_paren:
-          # Split before the dictionary value if we can't fit the whole
-          # dictionary on one line.
-          if not self._EachDictEntryFitsOnOneLine(opening):
+    if ((current.is_name or current.value in {'*', '**'}) and
+        previous.value == ','):
+      # If we have a function call within an argument list and it won't fit on
+      # the remaining line, but it will fit on a line by itself, then go ahead
+      # and split before the call.
+      opening = _GetOpeningBracket(current)
+      if (opening and opening.value == '(' and opening.previous_token and
+          (opening.previous_token.is_name or
+           opening.previous_token.value in {'*', '**'})):
+        is_func_call = False
+        token = current
+        while token:
+          if token.value == '(':
+            is_func_call = True
+            break
+          if (not (token.is_name or token.value in {'*', '**'}) and
+              token.value != '.'):
+            break
+          token = token.next_token
+
+        if is_func_call:
+          if not self._FitsOnLine(current, opening.matching_bracket):
             return True
 
+    pprevious = previous.previous_token
+    if (current.is_name and pprevious and pprevious.is_name and
+        previous.value == '('):
+      if (not self._FitsOnLine(previous, previous.matching_bracket) and
+          _IsFunctionCallWithArguments(current)):
+        # There is a function call, with more than 1 argument, where the first
+        # argument is itself a function call with arguments.  In this specific
+        # case, if we split after the first argument's opening '(', then the
+        # formatting will look bad for the rest of the arguments. E.g.:
+        #
+        #     outer_function_call(inner_function_call(
+        #         inner_arg1, inner_arg2),
+        #                         outer_arg1, outer_arg2)
+        #
+        # Instead, enforce a split before that argument to keep things looking
+        # good.
+        return True
+
     if (previous.OpensScope() and not current.OpensScope() and
         format_token.Subtype.SUBSCRIPT_BRACKET not in previous.subtypes):
-
       if not current.is_comment:
-        pprevious = previous.previous_token
         if pprevious and not pprevious.is_keyword and not pprevious.is_name:
           # We want to split if there's a comment in the container.
           token = current
@@ -242,19 +367,18 @@ class FormatDecisionState(object):
           # line and it's not a function call.
           if self._FitsOnLine(previous, previous.matching_bracket):
             return False
+        elif not self._FitsOnLine(previous, previous.matching_bracket):
+          if (self.column_limit - self.column) / float(self.column_limit) < 0.3:
+            # Try not to squish all of the arguments off to the right.
+            return current.next_token != previous.matching_bracket
       else:
         # Split after the opening of a container if it doesn't fit on the
         # current line or if it has a comment.
         if not self._FitsOnLine(previous, previous.matching_bracket):
           return True
 
-    if previous.value == '{':
-      closing = previous.matching_bracket
-      if (not self._FitsOnLine(previous, closing) and
-          closing.previous_token.value == ','):
-        self.stack[-1].split_before_closing_bracket = True
-        return True
-
+    ###########################################################################
+    # List Comprehension Splitting
     if (format_token.Subtype.COMP_FOR in current.subtypes and
         format_token.Subtype.COMP_FOR not in previous.subtypes):
       # Split at the beginning of a list comprehension.
@@ -270,24 +394,11 @@ class FormatDecisionState(object):
       if length + self.column > self.column_limit:
         return True
 
-    previous_previous_token = previous.previous_token
-    if (current.is_name and previous_previous_token and
-        previous_previous_token.is_name and previous.value == '('):
-      if not self._FitsOnLine(previous, previous.matching_bracket):
-        if _IsFunctionCallWithArguments(current):
-          # There is a function call, with more than 1 argument, where
-          # the first argument is itself a function call with arguments.
-          # In this specific case, if we split after the first argument's
-          # opening '(', then the formatting will look bad for the rest
-          # of the arguments. Instead, enforce a split before that
-          # argument to keep things looking good.
-          return True
-        elif current.OpensScope():
-          if not self._FitsOnLine(current, current.matching_bracket):
-            # There is a data literal that will need to be split and could mess
-            # up the formatting.
-            return True
-
+    ###########################################################################
+    # Original Formatting Splitting
+    # These checks rely upon the original formatting. This is in order to
+    # attempt to keep hand-written code in the same condition as it was before.
+    # However, this may cause the formatter to fail to be idempotent.
     if (style.Get('SPLIT_BEFORE_BITWISE_OPERATOR') and current.value in '&|' and
         previous.lineno < current.lineno):
       # Retain the split before a bitwise operator.
@@ -300,27 +411,6 @@ class FormatDecisionState(object):
       # original comments were on a separate line.
       return True
 
-    if current.is_name and previous.value == ',':
-      # If we have a function call within an argument list and it won't fit on
-      # the remaining line, but it will fit on a line by itself, then go ahead
-      # and split before the call.
-      opening = _GetOpeningBracket(current)
-      if (opening and opening.value == '(' and opening.previous_token and
-          opening.previous_token.is_name):
-        is_func_call = False
-        token = current
-        while token:
-          if token.value == '(':
-            is_func_call = True
-            break
-          if not token.is_name and token.value != '.':
-            break
-          token = token.next_token
-
-        if is_func_call:
-          if not self._FitsOnLine(current, opening.matching_bracket):
-            return True
-
     return False
 
   def AddTokenToState(self, newline, dry_run, must_split=False):
@@ -344,7 +434,7 @@ class FormatDecisionState(object):
     else:
       self._AddTokenOnCurrentLine(dry_run)
 
-    return self._MoveStateToNextToken() + penalty
+    return self.MoveStateToNextToken() + penalty
 
   def _AddTokenOnCurrentLine(self, dry_run):
     """Puts the token on the current line.
@@ -411,6 +501,7 @@ class FormatDecisionState(object):
          previous.previous_token.OpensScope())):
       self.stack[-1].closing_scope_indent = max(
           0, self.stack[-1].indent - style.Get('CONTINUATION_INDENT_WIDTH'))
+
       self.stack[-1].split_before_closing_bracket = True
 
     # Calculate the split penalty.
@@ -426,12 +517,19 @@ class FormatDecisionState(object):
 
     # Add a penalty for each increasing newline we add, but don't penalize for
     # splitting before an if-expression or list comprehension.
-    if not must_split and current.value not in {'if', 'for'}:
+    if current.value not in {'if', 'for'}:
       last = self.stack[-1]
       last.num_line_splits += 1
       penalty += (style.Get('SPLIT_PENALTY_FOR_ADDED_LINE_SPLIT') *
                   last.num_line_splits)
 
+    if current.OpensScope() and previous.OpensScope():
+      # Prefer to keep opening brackets coalesced (unless it's at the beginning
+      # of a function call).
+      pprev = previous.previous_token
+      if not pprev or not pprev.is_name:
+        penalty += 10
+
     return penalty + 10
 
   def _GetNewlineColumn(self):
@@ -473,7 +571,7 @@ class FormatDecisionState(object):
 
     return top_of_stack.indent
 
-  def _MoveStateToNextToken(self):
+  def MoveStateToNextToken(self):
     """Calculate format decision state information and move onto the next token.
 
     Before moving onto the next token, we first calculate the format decision
@@ -615,6 +713,7 @@ def _GetLengthOfSubtype(token, subtype, exclude=None):
 
 
 def _GetOpeningBracket(current):
+  """Get the opening bracket containing the current token."""
   if current.matching_bracket and not current.is_pseudo_paren:
     return current.matching_bracket
   while current:
@@ -674,7 +773,7 @@ class _ParenState(object):
     self.split_before_closing_bracket = False
     self.num_line_splits = 0
 
-  def __copy__(self):
+  def Clone(self):
     state = _ParenState(self.indent, self.last_space)
     state.closing_scope_indent = self.closing_scope_indent
     state.split_before_closing_bracket = self.split_before_closing_bracket
