@@ -24,6 +24,7 @@ from yapf.yapflib import style
 # TODO(morbo): Document the annotations in a centralized place. E.g., the
 # README file.
 UNBREAKABLE = 1000 * 1000
+NAMED_ASSIGN = 8500
 DOTTED_NAME = 4000
 VERY_STRONGLY_CONNECTED = 3500
 STRONGLY_CONNECTED = 3000
@@ -141,8 +142,9 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
     while index < len(node.children) - 1:
       child = node.children[index]
       if isinstance(child, pytree.Leaf) and child.value == '=':
-        _SetStronglyConnected(_FirstChildNode(node.children[index]))
-        _SetStronglyConnected(_FirstChildNode(node.children[index + 1]))
+        _SetSplitPenalty(_FirstChildNode(node.children[index]), NAMED_ASSIGN)
+        _SetSplitPenalty(
+            _FirstChildNode(node.children[index + 1]), NAMED_ASSIGN)
       index += 1
 
   def Visit_dotted_name(self, node):  # pylint: disable=invalid-name
@@ -178,6 +180,7 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
           _SetSplitPenalty(
               _FirstChildNode(node.children[1]), ONE_ELEMENT_ARGUMENT)
       elif (pytree_utils.NodeName(node.children[0]) == 'LSQB' and
+            len(node.children[1].children) > 2 and
             (name.endswith('_test') or name.endswith('_expr'))):
         _SetStronglyConnected(node.children[1].children[0])
         _SetStronglyConnected(node.children[1].children[2])
@@ -328,12 +331,12 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
     else:
       _IncreasePenalty(node, COMPARISON)
 
-  def Visit_star_expr(self, node):  # pylint: disable-invalid-name
+  def Visit_star_expr(self, node):  # pylint: disable=invalid-name
     # star_expr ::= '*' expr
     self.DefaultNodeVisit(node)
     _IncreasePenalty(node, STAR_EXPR)
 
-  def Visit_expr(self, node):  # pylint: disable-invalid-name
+  def Visit_expr(self, node):  # pylint: disable=invalid-name
     # expr ::= xor_expr ('|' xor_expr)*
     self.DefaultNodeVisit(node)
     _IncreasePenalty(node, EXPR)
@@ -389,6 +392,8 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
         if pytree_utils.NodeName(node.parent) == 'if_stmt':
           _SetSplitPenalty(node.children[-1], UNBREAKABLE)
         else:
+          if len(node.children) > 2:
+            _SetSplitPenalty(_FirstChildNode(node.children[1]), EXPR)
           _SetSplitPenalty(node.children[-1], ATOM)
     elif node.children[0].value in '[{' and len(node.children) == 2:
       # Keep empty containers together if we can.
