@@ -28,6 +28,67 @@ class BuganizerFixes(yapf_test_helper.YAPFTest):
   def setUpClass(cls):
     style.SetGlobalStyle(style.CreateChromiumStyle())
 
+  def testB37460004(self):
+    code = textwrap.dedent("""\
+        assert all(s not in (_SENTINEL, None) for s in
+                   nested_schemas), 'Nested schemas should never contain None/_SENTINEL'
+        """)
+    uwlines = yapf_test_helper.ParseAndUnwrap(code)
+    self.assertCodeEqual(code, reformatter.Reformat(uwlines))
+
+  def testB36806207(self):
+    unformatted_code = textwrap.dedent("""\
+        def _():
+          linearity_data = [[row] for row in [
+              "%.1f mm" % (np.mean(linearity_values["pos_error"]) * 1000.0),
+              "%.1f mm" % (np.max(linearity_values["pos_error"]) * 1000.0),
+              "%.1f mm" % (np.mean(linearity_values["pos_error_chunk_mean"]) * 1000.0),
+              "%.1f mm" % (np.max(linearity_values["pos_error_chunk_max"]) * 1000.0),
+              "%.1f deg" % math.degrees(np.mean(linearity_values["rot_noise"])),
+              "%.1f deg" % math.degrees(np.max(linearity_values["rot_noise"])),
+              "%.1f deg" % math.degrees(np.mean(linearity_values["rot_drift"])),
+              "%.1f deg" % math.degrees(np.max(linearity_values["rot_drift"])),
+              "%.1f%%" % (np.max(linearity_values["pos_discontinuity"]) * 100.0),
+              "%.1f%%" % (np.max(linearity_values["rot_discontinuity"]) * 100.0)
+          ]]
+        """)
+    expected_code = textwrap.dedent("""\
+        def _():
+          linearity_data = [
+              [row]
+              for row in [
+                  "%.1f mm" % (np.mean(linearity_values["pos_error"]) * 1000.0),
+                  "%.1f mm" % (np.max(linearity_values["pos_error"]) * 1000.0),
+                  "%.1f mm" % (
+                      np.mean(linearity_values["pos_error_chunk_mean"]) * 1000.0),
+                  "%.1f mm" % (
+                      np.max(linearity_values["pos_error_chunk_max"]) * 1000.0),
+                  "%.1f deg" % math.degrees(np.mean(linearity_values["rot_noise"])),
+                  "%.1f deg" % math.degrees(np.max(linearity_values["rot_noise"])),
+                  "%.1f deg" % math.degrees(np.mean(linearity_values["rot_drift"])),
+                  "%.1f deg" % math.degrees(np.max(linearity_values["rot_drift"])),
+                  "%.1f%%" % (np.max(linearity_values["pos_discontinuity"]) * 100.0),
+                  "%.1f%%" % (np.max(linearity_values["rot_discontinuity"]) * 100.0)
+              ]
+          ]
+        """)
+    uwlines = yapf_test_helper.ParseAndUnwrap(unformatted_code)
+    self.assertEqual(expected_code, reformatter.Reformat(uwlines))
+
+  def testB36215507(self):
+    code = textwrap.dedent("""\
+        class X():
+
+          def _():
+            aaaaaaaaaaaaa._bbbbbbbbbbbbbbbbbbbbbbbbbbbbbb(
+                mmmmmmmmmmmmm, nnnnn, ooooooooo,
+                _(ppppppppppppppppppppppppppppppppppppp),
+                *(qqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqq),
+                **(qqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqq))
+        """)
+    uwlines = yapf_test_helper.ParseAndUnwrap(code)
+    self.assertCodeEqual(code, reformatter.Reformat(uwlines))
+
   def testB35212469(self):
     unformatted_code = textwrap.dedent("""\
         def _():
