@@ -365,7 +365,7 @@ class CommandLineTest(unittest.TestCase):
         stderr=subprocess.PIPE)
     reformatted_code, stderrdata = p.communicate(unformatted.encode('utf-8'))
     self.assertEqual(stderrdata, b'')
-    self.assertEqual(reformatted_code.decode('utf-8'), expected)
+    self.assertMultiLineEqual(reformatted_code.decode('utf-8'), expected)
 
   def testUnicodeEncodingPipedToFile(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -492,7 +492,10 @@ class CommandLineTest(unittest.TestCase):
         suffix='.py', dirname=self.test_tmpdir) as (out, _):
       with utils.TempFileContents(
           self.test_tmpdir, unformatted_code, suffix='.py') as filepath:
-        subprocess.check_call(YAPF_BINARY + ['--diff', filepath], stdout=out)
+        try:
+          subprocess.check_call(YAPF_BINARY + ['--diff', filepath], stdout=out)
+        except subprocess.CalledProcessError as e:
+          self.assertEqual(e.returncode, 1)  # Indicates the text changed.
 
   def testReformattingSpecificLines(self):
     unformatted_code = textwrap.dedent("""\
@@ -775,6 +778,8 @@ class CommandLineTest(unittest.TestCase):
         """)
     expected_formatted_code = textwrap.dedent("""\
         a = line_to_format
+
+
         def f():
             x = y + 42; z = n * 42
             if True: a += 1 ; b += 1 ; c += 1
@@ -797,6 +802,8 @@ class CommandLineTest(unittest.TestCase):
         ''')
     expected_formatted_code = textwrap.dedent('''\
         foo = 42
+
+
         def f():
             email_text += """<html>This is a really long docstring that goes over the column limit and is multi-line.<br><br>
         <b>Czar: </b>"""+despot["Nicholas"]+"""<br>
@@ -1010,20 +1017,22 @@ class CommandLineTest(unittest.TestCase):
 
   def testCoalesceBrackets(self):
     unformatted_code = textwrap.dedent("""\
-       some_long_function_name_foo({
-           'first_argument_of_the_thing': id,
-           'second_argument_of_the_thing': "some thing"}
-           )""")
+       some_long_function_name_foo(
+           {
+               'first_argument_of_the_thing': id,
+               'second_argument_of_the_thing': "some thing"
+           }
+       )""")
     expected_formatted_code = textwrap.dedent("""\
        some_long_function_name_foo({
            'first_argument_of_the_thing': id,
-           'second_argument_of_the_thing': "some thing"})
+           'second_argument_of_the_thing': "some thing"
+       })
        """)
     with utils.NamedTempFile(dirname=self.test_tmpdir, mode='w') as (f, name):
       f.write(
           textwrap.dedent(u'''\
           [style]
-          based_on_style = facebook
           column_limit=82
           coalesce_brackets = True
           '''))
@@ -1041,6 +1050,7 @@ class CommandLineTest(unittest.TestCase):
         """)
     expected_formatted_code = textwrap.dedent("""\
         def foo():
+
           def bar():
             return {msg_id: author for author, msg_id in reader}
         """)
